@@ -24,4 +24,16 @@ define('INSTALL_PATH', realpath(__DIR__ . '/..') . '/' );
 require INSTALL_PATH.'program/include/clisetup.php';
 
 $rcmail = rcube::get_instance();
+
+$session_driver   = $rcmail->config->get('session_storage', 'db');
+$session_lifetime = $rcmail->config->get('session_lifetime', 0) * 60 * 2;
+
+// Clean expired SQL sessions
+if ($session_driver == 'db' && $session_lifetime) {
+    $db = $rcmail->get_dbh();
+    $db->query("DELETE FROM " . $db->table_name('session')
+        . " WHERE changed < " . $db->now(-$session_lifetime));
+}
+
+// Clean caches and temp directory
 $rcmail->gc();
