@@ -39,7 +39,7 @@ function rcube_text_editor(config, id)
     abs_url = location.href.replace(/[?#].*$/, '').replace(/\/$/, ''),
     conf = {
       selector: '#' + ($('#' + id).is('.mce_editor') ? id : 'fake-editor-id'),
-      cache_suffix: 's=4010700',
+      cache_suffix: 's=4020700',
       theme: 'modern',
       language: config.lang,
       content_css: rcmail.assets_path('program/js/tinymce/roundcube/content.css'),
@@ -65,6 +65,18 @@ function rcube_text_editor(config, id)
     }
   }
 
+  // secure spellchecker requests with Roundcube token
+  // Note: must be registered only once (#1490311)
+  if (!tinymce.registered_request_token) {
+    tinymce.registered_request_token = true;
+    tinymce.util.XHR.on('beforeSend', function(e) {
+      e.xhr.setRequestHeader('X-Roundcube-Request', rcmail.env.request_token);
+      // Fix missing lang parameter on addToDictionary request (#1490634)
+      if (e.settings && e.settings.data && /^method=addToDictionary/.test(e.settings.data) && !/&lang=/.test(e.settings.data))
+        e.settings.data += '&lang=' + ref.editor.plugins.spellchecker.getLanguage();
+    });
+  }
+
   // minimal editor
   if (config.mode == 'identity') {
     $.extend(conf, {
@@ -79,11 +91,11 @@ function rcube_text_editor(config, id)
   // full-featured editor
   else {
     $.extend(conf, {
-      plugins: 'autolink charmap code colorpicker directionality emoticons link image media nonbreaking'
-        + ' paste table tabfocus textcolor searchreplace' + (config.spellcheck ? ' spellchecker' : ''),
+      plugins: 'autolink charmap code colorpicker directionality link image media nonbreaking'
+        + ' paste table tabfocus textcolor searchreplace spellchecker',
       toolbar: 'bold italic underline | alignleft aligncenter alignright alignjustify'
         + ' | bullist numlist outdent indent ltr rtl blockquote | forecolor backcolor | fontselect fontsizeselect'
-        + ' | link unlink table | emoticons charmap image media | code searchreplace undo redo',
+        + ' | link unlink table | $extra charmap image media | code searchreplace undo redo',
       spellchecker_rpc_url: abs_url + '/?_task=utils&_action=spell_html&_remote=1',
       spellchecker_language: rcmail.env.spell_lang,
       accessibility_focus: false,
@@ -93,6 +105,26 @@ function rcube_text_editor(config, id)
     });
   }
 
+  // add TinyMCE plugins/buttons from Roundcube plugin
+  $.each(config.extra_plugins || [], function() {
+    if (conf.plugins.indexOf(this) < 0)
+      conf.plugins = conf.plugins + ' ' + this;
+  });
+  $.each(config.extra_buttons || [], function() {
+    if (conf.toolbar.indexOf(this) < 0)
+      conf.toolbar = conf.toolbar.replace('$extra', '$extra ' + this);
+  });
+
+  // disable TinyMCE plugins/buttons from Roundcube plugin
+  $.each(config.disabled_plugins || [], function() {
+    conf.plugins = conf.plugins.replace(this, '');
+  });
+  $.each(config.disabled_buttons || [], function() {
+    conf.toolbar = conf.toolbar.replace(this, '');
+  });
+
+  conf.toolbar = conf.toolbar.replace('$extra', '').replace(/\|\s+\|/g, '|');
+
   // support external configuration settings e.g. from skin
   if (window.rcmail_editor_settings)
     $.extend(conf, window.rcmail_editor_settings);
@@ -107,10 +139,6 @@ function rcube_text_editor(config, id)
     ed.on('keypress', function() {
       rcmail.compose_type_activity++;
     });
-    // secure spellchecker requests with Roundcube token
-    tinymce.util.XHR.on('beforeSend', function(e) {
-      e.xhr.setRequestHeader('X-Roundcube-Request', rcmail.env.request_token);
-    });
   };
 
   // textarea identifier
@@ -129,6 +157,15 @@ function rcube_text_editor(config, id)
       return;
     }
 
+    var area = $('#' + this.id),
+      height = $('div.mce-toolbar-grp:first', area.parent()).height();
+
+    // the editor might be still not fully loaded, making the editing area
+    // inaccessible, wait and try again (#1490310)
+    if (height > 200 || height > area.height()) {
+      return setTimeout(function () { ref.init_callback(event); }, 300);
+    }
+
     var css = {},
       elem = rcube_find_object('_from'),
       fe = rcmail.env.compose_focus_elem;
@@ -149,18 +186,16 @@ function rcube_text_editor(config, id)
 
       // Focus previously focused element
       if (fe && fe.id != this.id) {
-        // use setTimeout() for IE9 (#1488541)
-        window.setTimeout(function() {
-          window.focus(); // for WebKit (#1486674)
-          fe.focus();
-        }, 10);
+        window.focus(); // for WebKit (#1486674)
+        fe.focus();
+        rcmail.env.compose_focus_elem = null;
       }
     }
 
     // set tabIndex and set focus to element that was focused before
-    this.tabindex(fe && fe.id == this.id);
+    ref.tabindex(fe && fe.id == ref.id);
     // Trigger resize (needed for proper editor resizing in some browsers)
-    window.setTimeout(function() { $(window).resize(); }, 100);
+    $(window).resize();
   };
 
   // set tabIndex on tinymce editor
@@ -168,20 +203,19 @@ function rcube_text_editor(config, id)
   {
     if (rcmail.env.task == 'mail' && this.editor) {
       var textarea = this.editor.getElement(),
+        body = this.editor.getBody(),
         node = this.editor.getContentAreaContainer().childNodes[0];
 
       if (textarea && node)
         node.tabIndex = textarea.tabIndex;
-      if (focus)
-        this.editor.getBody().focus();
 
       // find :prev and :next elements to get focus when tabbing away
       if (textarea.tabIndex > 0) {
         var x = null,
-          editor = this.editor,
           tabfocus_elements = [':prev',':next'],
           el = tinymce.DOM.select('*[tabindex='+textarea.tabIndex+']:not(iframe)');
-        tinymce.each(el, function(e, i) { if (e.id == editor.id) { x = i; return false; } });
+
+        tinymce.each(el, function(e, i) { if (e.id == ref.id) { x = i; return false; } });
         if (x !== null) {
           if (el[x-1] && el[x-1].id) {
             tabfocus_elements[0] = el[x-1].id;
@@ -189,9 +223,16 @@ function rcube_text_editor(config, id)
           if (el[x+1] && el[x+1].id) {
             tabfocus_elements[1] = el[x+1].id;
           }
-          editor.settings.tabfocus_elements = tabfocus_elements.join(',');
+          this.editor.settings.tabfocus_elements = tabfocus_elements.join(',');
         }
       }
+
+      // ContentEditable reset fixes invisible cursor issue in Firefox < 25
+      if (bw.mz && bw.vendver < 25)
+        $(body).prop('contenteditable', false).prop('contenteditable', true);
+
+      if (focus)
+        body.focus();
     }
   };
 
@@ -226,14 +267,13 @@ function rcube_text_editor(config, id)
         input.val(data);
         tinymce.execCommand('mceAddEditor', false, ref.id);
 
-        setTimeout(function() {
-          if (ref.editor) {
-            if (rcmail.env.default_font)
-              $(ref.editor.getBody()).css('font-family', rcmail.env.default_font);
-            // #1486593
-            ref.tabindex(true);
-          }
-        }, 500);
+        if (ref.editor) {
+          var body = $(ref.editor.getBody());
+          // #1486593
+          ref.tabindex(true);
+          // put cursor on start of the compose body
+          ref.editor.selection.setCursorLocation(body.children().first().get(0));
+        }
       };
 
       // convert to html
@@ -271,6 +311,7 @@ function rcube_text_editor(config, id)
           data = data.replace(sig_mark, "\n" + signature.text);
 
         input.val(data).focus();
+        rcmail.set_caret_pos(input.get(0), 0);
       };
 
       // convert html to text
@@ -409,7 +450,6 @@ function rcube_text_editor(config, id)
 
     // get selected text from tinymce editor
     if (ed) {
-      ed.getWin().focus(); // correct focus in IE & Chrome
       if (args.selection)
         text = ed.selection.getContent({format: args.format});
 
@@ -446,7 +486,7 @@ function rcube_text_editor(config, id)
   // change user signature text
   this.change_signature = function(id, show_sig)
   {
-    var cursor_pos, p = -1,
+    var position_element, cursor_pos, p = -1,
       input_message = $('#' + this.id),
       message = input_message.val(),
       sig = rcmail.env.identity;
@@ -461,37 +501,43 @@ function rcube_text_editor(config, id)
         if (p >= 0)
           message = message.substring(0, p) + message.substring(p+sig.length, message.length);
       }
+
       // add the new signature string
       if (show_sig && rcmail.env.signatures && rcmail.env.signatures[id]) {
         sig = rcmail.env.signatures[id].text;
         sig = sig.replace(/\r\n/g, '\n');
 
-        if (rcmail.env.top_posting) {
-          if (p >= 0) { // in place of removed signature
-            message = message.substring(0, p) + sig + message.substring(p, message.length);
-            cursor_pos = p - 1;
-          }
-          else if (!message) { // empty message
-            cursor_pos = 0;
-            message = '\n\n' + sig;
-          }
-          else if (pos = rcmail.get_caret_pos(input_message.get(0))) { // at cursor position
+        // in place of removed signature
+        if (p >= 0) {
+          message = message.substring(0, p) + sig + message.substring(p, message.length);
+          cursor_pos = p - 1;
+        }
+        // empty message or new-message mode
+        else if (!message || !rcmail.env.compose_mode) {
+          cursor_pos = message.length;
+          message += '\n\n' + sig;
+        }
+        else if (rcmail.env.top_posting && !rcmail.env.sig_below) {
+          // at cursor position
+          if (pos = rcmail.get_caret_pos(input_message.get(0))) {
             message = message.substring(0, pos) + '\n' + sig + '\n\n' + message.substring(pos, message.length);
             cursor_pos = pos;
           }
-          else { // on top
-            cursor_pos = 0;
+          // on top
+          else {
             message = '\n\n' + sig + '\n\n' + message.replace(/^[\r\n]+/, '');
+            cursor_pos = 0;
           }
         }
         else {
           message = message.replace(/[\r\n]+$/, '');
-          cursor_pos = !rcmail.env.top_posting && message.length ? message.length+1 : 0;
+          cursor_pos = !rcmail.env.top_posting && message.length ? message.length + 1 : 0;
           message += '\n\n' + sig;
         }
       }
-      else
+      else {
         cursor_pos = rcmail.env.top_posting ? 0 : message.length;
+      }
 
       input_message.val(message);
 
@@ -503,35 +549,38 @@ function rcube_text_editor(config, id)
 
       // Append the signature as a div within the body
       if (!sigElem) {
-        var body = this.editor.getBody(),
-          doc = this.editor.getDoc();
+        var body = this.editor.getBody();
 
-        sigElem = doc.createElement('div');
-        sigElem.setAttribute('id', '_rc_sig');
+        sigElem = $('<div id="_rc_sig"></div>').get(0);
 
-        if (rcmail.env.top_posting) {
-          // if no existing sig and top posting then insert at caret pos
+        // insert at start or at cursor position in top-posting mode
+        // (but not if the content is empty and not in new-message mode)
+        if (rcmail.env.top_posting && !rcmail.env.sig_below
+          && rcmail.env.compose_mode && (body.childNodes.length > 1 || $(body).text())
+        ) {
           this.editor.getWin().focus(); // correct focus in IE & Chrome
 
           var node = this.editor.selection.getNode();
-          if (node.nodeName == 'BODY') {
-            // no real focus, insert at start
-            body.insertBefore(sigElem, body.firstChild);
-            body.insertBefore(doc.createElement('br'), body.firstChild);
-          }
-          else {
-            body.insertBefore(sigElem, node.nextSibling);
-            body.insertBefore(doc.createElement('br'), node.nextSibling);
-          }
+
+          $(sigElem).insertBefore(node.nodeName == 'BODY' ? body.firstChild : node.nextSibling);
+          $('<p>').append($('<br>')).insertBefore(sigElem);
         }
         else {
           body.appendChild(sigElem);
+          position_element = rcmail.env.top_posting && rcmail.env.compose_mode ? body.firstChild : $(sigElem).prev();
         }
       }
 
-      if (rcmail.env.signatures[id]) {
-        sigElem.innerHTML = rcmail.env.signatures[id].html;
-      }
+      sigElem.innerHTML = rcmail.env.signatures[id] ? rcmail.env.signatures[id].html : '';
+    }
+    else if (!rcmail.env.top_posting) {
+      position_element = $(this.editor.getBody()).children().last();
+    }
+
+    // put cursor before signature and scroll the window
+    if (this.editor && position_element && position_element.length) {
+      this.editor.selection.setCursorLocation(position_element.get(0));
+      this.editor.getWin().scroll(0, position_element.offset().top);
     }
   };
 
@@ -556,7 +605,7 @@ function rcube_text_editor(config, id)
 
     // open image selector dialog
     dialog = this.editor.windowManager.open({
-      title: rcmail.gettext('select' + type),
+      title: rcmail.get_label('select' + type),
       width: 500,
       height: 300,
       html: '<div id="image-selector-list"><ul></ul></div>'
@@ -583,7 +632,7 @@ function rcube_text_editor(config, id)
       .text($('div.hint', rcmail.gui_objects.uploadform).text()));
 
     // init upload button
-    elem = $('#image-upload-button').append($('<span>').text(rcmail.gettext('add' + type)));
+    elem = $('#image-upload-button').append($('<span>').text(rcmail.get_label('add' + type)));
     cancel = elem.parents('.mce-panel').find('button:last').parent();
 
     // we need custom Tab key handlers, until we find out why
@@ -626,7 +675,7 @@ function rcube_text_editor(config, id)
 
       rcmail.gui_objects.filedrop = $('#image-selector-form');
       rcmail.gui_objects.filedrop.addClass('droptarget')
-        .bind('dragover dragleave', function(e) {
+        .on('dragover dragleave', function(e) {
           e.preventDefault();
           e.stopPropagation();
           $(this)[(e.type == 'dragover' ? 'addClass' : 'removeClass')]('hover');
